#!/bin/bash
# Backup Progress Calculator for OrangeBOX by NLD
# Version: 0.2 (2018/03/05)
#
# Run it as unprivileged user from cron
# * * * * * /home/orange/backup_progress.sh &> /dev/null
#
# This script is responsible only for displaying data on the LCD, it communicates
# with the main program indirectly through position and lock files.

BACKUP_DRIVE="/dev/mapper/backup"
VFILE="$HOME/start.pos"  # disk usage at the beginning of the backup
TFILE="$HOME/trans.size" # overall precalculated transfer size
BFILE="$HOME/backup.lck" # determines started state
FFILE="$HOME/backup.fin" # determines finished state
LFILE1="$HOME/lcd1.bar"  # LCD progress indicator data
LFILE2="$HOME/lcd2.bar"  # LCD progress indicator data
SHUTDOWN="1"             # If 1 it will initiate another script which shuts down the box at the end of backup
BACKUP_CURRENT="0"       # Has to be initialized but will be calculated
DRIVE_SIZE="<SET THIS VALUE>" # Drive size in bytes (secondary check)
LCD="sudo /bin/lcd"
function is_mount() {
                grep -q "$1" /proc/mounts
                status=$?
}

function red() {
 sudo /bin/lcdcolor red
}
function green() {
 sudo /bin/lcdcolor green
}
function blue() {
 sudo /bin/lcdcolor blue
}

# Clear state (set on boot by bootup_display.sh). No backup is in progress, do NOT mess up the
# status LCD. Only display progress in case of there is an ongoing backup => No start file AND No fin file = quit
if [ ! -f $BFILE ] && [ ! -f $FFILE  ]; then
  exit 1
fi

# If the backup finished this script will display that and remove the locks
# not to be able to run again until the next initiation.
if [ -f $FFILE ]; then
   green
   $LCD "   Backup   " "** Completed **"
   echo "Backup Completed"
   rm -rf $BFILE $TFILE $FFILE $LFILE1 $LFILE2 $VFILE # Backup finished clean up

   if [ $SHUTDOWN == "1" ]; then
        echo "Executing shutdown script..."
        /home/orange/shutdown.sh &
   fi
   exit 0
fi

# From this point the script ONLY executes if there is an ONGOING backup
# therefore all errors will be printed out to the LCD and cause the script
# to abort however it will not remove the backup.lck file so will go in here
# over and over again and evaulate the conditions.

is_mount $BACKUP_DRIVE
  if [ $status -ne 0 ]; then
          red
          $LCD "ERR: Backup drive" "is not mounted!"
          echo "Backup drive is not mounted"
          exit 1
  fi

if [ ! -s $TFILE ]; then
          red
          $LCD "ERR: transfile" "is empty"
          echo "Transport size calculation file is empty."
          exit 1
fi

BACKUP_OVERALL=$(head -1 $TFILE |tr -d '\n')

if [ -z $BACKUP_OVERALL ]; then
          red
          $LCD "ERR: Size readback" "from server invalid"
          echo "Backup overall size readback is invalid 1"
          exit 1
fi

if ! [[ $BACKUP_OVERALL =~ ^-?[0-9]+$ ]] ; then
          red
          $LCD "ERR: Size readback" "from server invalid"
          echo "Backup overall size readback is invalid 2"
          exit 1
fi


if [ $BACKUP_OVERALL -le 0 ]; then
          red
          $LCD "ERR: Size readback" "Size is too small"
          echo "Overall backup size is too small"
          exit 1
fi

# Query filesystem information in bytes
SPACE_ALL=$(df -B1 | grep $BACKUP_DRIVE | awk '{ print $2 }')
SPACE_USED=$(df -B1 | grep $BACKUP_DRIVE | awk '{ print $3 }')
SPACE_FREE=$(df -B1 | grep $BACKUP_DRIVE | awk '{ print $4 }')

if [ $SPACE_ALL -ne $DRIVE_SIZE ]; then
          red
          $LCD "ERR: drive size" "Space mismatch"
          echo "The mounted hard drive is not identical with the hardcoded value."
          exit 1
fi

if [ ! -f $VFILE ]; then
        START_POS=$SPACE_USED
        echo "START_POS=$SPACE_USED" > $VFILE

else
        source $VFILE
        BACKUP_CURRENT=$(expr $SPACE_USED - $START_POS)
fi

if [ $BACKUP_OVERALL -ge $SPACE_FREE ]; then
        red
        $LCD "ERR: Backup size" "exceeds disk space!"
        echo "Backup exceeds free space, not performing backup"
        exit 1
fi

#Blue light from here
        blue
PROGRESS=$(( ($BACKUP_CURRENT * 100) / $BACKUP_OVERALL ))

BACKUP_OVERALL_MB=$(expr $BACKUP_OVERALL / 1024 / 1024)
BACKUP_CURRENT_MB=$(expr $BACKUP_CURRENT / 1024 / 1024)
BACKUP_OVERALL_GB=$(expr $BACKUP_OVERALL_MB / 1024)
BACKUP_CURRENT_GB=$(expr $BACKUP_CURRENT_MB / 1024)

if [ $BACKUP_OVERALL_GB -gt 1 ]; then
   LCD_OVERALL="$BACKUP_OVERALL_GB GB"
else
   LCD_OVERALL="$BACKUP_OVERALL_MB MB"
fi

if [ $BACKUP_CURRENT_GB -gt 1 ]; then
   LCD_CURRENT="$BACKUP_CURRENT_GB GB"
else
   LCD_CURRENT="$BACKUP_CURRENT_MB MB"
fi

# Maps the percentage on a scale of 20 COL LCD
  LCD_BAR=$(expr $PROGRESS \* 20 / 100)
  rm -rf $LFILE1 $LFILE2

# LCD Display

  echo "$LCD_CURRENT / $LCD_OVERALL" | cut -b -19 > $LFILE1
  for ((i=0;i<=$LCD_BAR;i++))
        do
          echo -n "#" >> $LFILE2
        done

 sudo /bin/lcd "$(head -1 $LFILE1)"  $(head -1 $LFILE2 | cut -b -19)

# Console Display
echo "-------------------------------------------------------"
echo "Space all: $SPACE_ALL bytes => $(expr $SPACE_ALL / 1024 / 1024) MB => $(expr $SPACE_ALL / 1024 / 1024 / 1024) GB"
echo "Space used: $START_POS bytes => $(expr $START_POS / 1024 / 1024) MB => $(expr $SPACE_ALL / 1024 / 1024 / 1024) GB"
echo "Space free: $SPACE_FREE bytes => $(expr $SPACE_FREE / 1024 / 1024) MB => $(expr $SPACE_ALL / 1024 / 1024 / 1024) GB"
echo "-------------------------------------------------------"
echo "Backup overall: $BACKUP_OVERALL bytes => $BACKUP_OVERALL_MB MB => $BACKUP_OVERALL_GB GB"
echo "Backup completed: $BACKUP_CURRENT bytes => $BACKUP_CURRENT_MB MB => $BACKUP_CURRENT_GB GB"
echo "Progress: $PROGRESS %"
