#!/bin/bash

###
# OrangeBox Remote Script by NLD
#
# Offsite copy to OrangeBox.
# This script is not ZFS snapshot aware, it just rsyncs the whole current backup drive as it is.
###

BACKUP_CLIENT=orangebox
BACKUP_HOST=orangebox.yourdomain.lan
BACKUP_PORT=22
SSH_CMD="/usr/bin/ssh -o UserKnownHostsFile=/mnt/backup/scripts/hosts -i /mnt/backup/scripts/backup.key -p ${BACKUP_PORT}"
BACKUPROOT="/mnt/backup"
RSYNC_BIN="/usr/local/bin/rsync"
BUF=/tmp/${BACKUP_HOST}_filter
DISK="/dev/disk/by-id/<YOUR DRIVE ID HERE>"

cat << EOF > $BUF
+ dir1toinclude/
+ dir2toinclude/
+ dir3toinclude/
- dir4toexclude/
- /*
EOF

if ping -c 1 -W 2 ${BACKUP_HOST} > /dev/null; then
 echo "[backup] Performing backup"
else
 echo "[backup] OrangeBOX is OFFLINE, exiting."
 exit 1
fi

$SSH_CMD root@${BACKUP_HOST} "rm -rf /tmp/contmsg.txt && chmod 755 /usr/bin/rsync"

$RSYNC_BIN -avz --numeric-ids --delete -r --delete-excluded -e "${SSH_CMD}" /mnt/backup/scripts/orangebox.key root@${BACKUP_HOST}:/dev/shm/orangebox

$SSH_CMD root@${BACKUP_HOST} "cryptsetup luksOpen --key-file=/dev/shm/orangebox $DISK backup &> /tmp/contmsg.txt"
$SSH_CMD root@${BACKUP_HOST} "if grep -qi denied /tmp/contmsg.txt; then chmod 000 /usr/bin/rsync; fi"
$SSH_CMD root@${BACKUP_HOST} "shred -u /dev/shm/orangebox"
$SSH_CMD root@${BACKUP_HOST} "mount /dev/mapper/backup /mnt/backup/"

echo ${BACKUP_CLIENT} START TIME
date

$SSH_CMD root@${BACKUP_HOST} 'su orange -c "rm -f /home/orange/backup.fin && touch /home/orange/backup.lck"'

#Size Precalculation (dry)
$RSYNC_BIN -n --numeric-ids --delete  -r --delete-excluded -f "merge ${BUF}" --stats -e "${SSH_CMD}" $BACKUPROOT/ root@${BACKUP_HOST}:$BACKUPROOT/ | grep 'Total transferred file size' | awk '{ print $5 }' | tr -d ',' > /tmp/orange.size

#Size file transfer
$RSYNC_BIN -avz -e "${SSH_CMD}" /tmp/orange.size root@${BACKUP_HOST}:/home/orange/trans.size

# # # MAIN BACKUP LOOP # # #
for ((i=0;i<100;i++)); do
$RSYNC_BIN -avz --timeout=240 --numeric-ids --delete -r --delete-excluded --delete-after -f "merge ${BUF}" --stats -e "${SSH_CMD}" $BACKUPROOT/ root@${BACKUP_HOST}:$BACKUPROOT/

retVal=$?
if [ $retVal -eq 0 ]; then
    echo "Completed Successfully."
    break
fi
done

echo ${BACKUP_CLIENT} STOP TIME
date

$SSH_CMD root@${BACKUP_HOST} 'su orange -c "rm -f /home/orange/backup.lck && touch /home/orange/backup.fin"'
$SSH_CMD root@${BACKUP_HOST} "umount /mnt/backup"
$SSH_CMD root@${BACKUP_HOST} "cryptsetup luksClose backup"
